package cn.com.ctop.toutiao.modules.material.controller;

import cn.com.ctop.common.module.utils.QueryGenerator;
import cn.com.ctop.toutiao.modules.material.entity.BytedanceAccountCommonSlogen;
import cn.com.ctop.toutiao.modules.material.service.IBytedanceAccountCommonSlogenService;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.jeecg.common.api.vo.Result;
import org.jeecg.common.util.oConvertUtils;
import org.jeecgframework.poi.excel.ExcelImportUtil;
import org.jeecgframework.poi.excel.def.NormalExcelConstants;
import org.jeecgframework.poi.excel.entity.ExportParams;
import org.jeecgframework.poi.excel.entity.ImportParams;
import org.jeecgframework.poi.excel.view.JeecgEntityExcelView;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.URLDecoder;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * 头条账户通用文案数据
 * @author jeecg-boot
 * @date   2021-05-24
 * @version V1.0
 */
@Slf4j
@Api(tags="头条账户通用文案数据")
@RestController
@RequestMapping("/ctop/bytedanceAccountCommonSlogen")
public class BytedanceAccountCommonSlogenController {
	@Autowired
	private IBytedanceAccountCommonSlogenService bytedanceAccountCommonSlogenService;

	/**
	  * 分页列表查询
	 * @param bytedanceAccountCommonSlogen
	 * @param pageNo
	 * @param pageSize
	 * @param req
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-分页列表查询", notes="头条账户通用文案数据-分页列表查询")
	@GetMapping(value = "/list")
	public Result<IPage<BytedanceAccountCommonSlogen>> queryPageList(BytedanceAccountCommonSlogen bytedanceAccountCommonSlogen,
																	 @RequestParam(name="pageNo", defaultValue="1") Integer pageNo,
																	 @RequestParam(name="pageSize", defaultValue="10") Integer pageSize,
																	 HttpServletRequest req) {
		Result<IPage<BytedanceAccountCommonSlogen>> result = new Result<>();
		QueryWrapper<BytedanceAccountCommonSlogen> queryWrapper = QueryGenerator.initQueryWrapper(bytedanceAccountCommonSlogen, req.getParameterMap());
		Page<BytedanceAccountCommonSlogen> page = new Page<>(pageNo, pageSize);
		IPage<BytedanceAccountCommonSlogen> pageList = bytedanceAccountCommonSlogenService.page(page, queryWrapper);
		result.setSuccess(true);
		result.setResult(pageList);
		return result;
	}

	/**
	  *   添加
	 * @param bytedanceAccountCommonSlogen
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-添加", notes="头条账户通用文案数据-添加")
	@PostMapping(value = "/add")
	public Result<BytedanceAccountCommonSlogen> add(@RequestBody BytedanceAccountCommonSlogen bytedanceAccountCommonSlogen) {
		Result<BytedanceAccountCommonSlogen> result = new Result<>();
		try {
			bytedanceAccountCommonSlogenService.save(bytedanceAccountCommonSlogen);
			result.success("添加成功！");
		} catch (Exception e) {
			log.error(e.getMessage(),e);
			result.error500("操作失败");
		}
		return result;
	}

	/**
	  *  编辑
	 * @param bytedanceAccountCommonSlogen
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-编辑", notes="头条账户通用文案数据-编辑")
	@PutMapping(value = "/edit")
	public Result<BytedanceAccountCommonSlogen> edit(@RequestBody BytedanceAccountCommonSlogen bytedanceAccountCommonSlogen) {
		Result<BytedanceAccountCommonSlogen> result = new Result<>();
		BytedanceAccountCommonSlogen bytedanceAccountCommonSlogenEntity = bytedanceAccountCommonSlogenService.getById(bytedanceAccountCommonSlogen.getId());
		if(bytedanceAccountCommonSlogenEntity==null) {
			result.error500("未找到对应实体");
		}else {
			boolean ok = bytedanceAccountCommonSlogenService.updateById(bytedanceAccountCommonSlogen);
			if(ok) {
				result.success("修改成功!");
			}
		}

		return result;
	}

	/**
	  *   通过id删除
	 * @param id
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-通过id删除", notes="头条账户通用文案数据-通过id删除")
	@DeleteMapping(value = "/delete")
	public Result<?> delete(@RequestParam(name="id") String id) {
		try {
			bytedanceAccountCommonSlogenService.removeById(id);
		} catch (Exception e) {
			log.error("删除失败",e.getMessage());
			return Result.error("删除失败!");
		}
		return Result.ok("删除成功!");
	}

	/**
	 *  批量删除
	 * @param ids
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-批量删除", notes="头条账户通用文案数据-批量删除")
	@DeleteMapping(value = "/deleteBatch")
	public Result<BytedanceAccountCommonSlogen> deleteBatch(@RequestParam(name="ids") String ids) {
		Result<BytedanceAccountCommonSlogen> result = new Result<>();
		if(ids==null || "".equals(ids.trim())) {
			result.error500("参数不识别！");
		}else {
			this.bytedanceAccountCommonSlogenService.removeByIds(Arrays.asList(ids.split(",")));
			result.success("删除成功!");
		}
		return result;
	}

	/**
	  * 通过id查询
	 * @param id
	 * @return
	 */
	@ApiOperation(value="头条账户通用文案数据-通过id查询", notes="头条账户通用文案数据-通过id查询")
	@GetMapping(value = "/queryById")
	public Result<BytedanceAccountCommonSlogen> queryById(@RequestParam(name="id",required=true) String id) {
		Result<BytedanceAccountCommonSlogen> result = new Result<>();
		BytedanceAccountCommonSlogen bytedanceAccountCommonSlogen = bytedanceAccountCommonSlogenService.getById(id);
		if(bytedanceAccountCommonSlogen==null) {
			result.error500("未找到对应实体");
		}else {
			result.setResult(bytedanceAccountCommonSlogen);
			result.setSuccess(true);
		}
		return result;
	}

  /**
   * 通过excel导入数据
   *
   * @param request
   * @param response
   * @return
   */
  @RequestMapping(value = "/importExcel", method = RequestMethod.POST)
  public Result<?> importExcel(HttpServletRequest request, HttpServletResponse response) {
      MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
      Map<String, MultipartFile> fileMap = multipartRequest.getFileMap();
      for (Map.Entry<String, MultipartFile> entity : fileMap.entrySet()) {
          MultipartFile file = entity.getValue();
          ImportParams params = new ImportParams();
          params.setTitleRows(2);
          params.setHeadRows(1);
          params.setNeedSave(true);
          try {
              List<BytedanceAccountCommonSlogen> listBytedanceAccountCommonSlogens = ExcelImportUtil.importExcel(file.getInputStream(), BytedanceAccountCommonSlogen.class, params);
              bytedanceAccountCommonSlogenService.saveBatch(listBytedanceAccountCommonSlogens);
              return Result.ok("文件导入成功！数据行数:" + listBytedanceAccountCommonSlogens.size());
          } catch (Exception e) {
              log.error(e.getMessage(),e);
              return Result.error("文件导入失败:"+e.getMessage());
          } finally {
              try {
                  file.getInputStream().close();
              } catch (IOException e) {
                  e.printStackTrace();
              }
          }
      }
      return Result.ok("文件导入失败！");
  }

}
